<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2006-07-20
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
// The parent class
require_once 'plugins/docman/lib/om/BaseDocmanFilePeer.php';

// The object class
include_once 'plugins/docman/lib/DocmanFile.php';

/**
 * The skeleton for this class was autogenerated by Propel  *  You should add additional methods to this class to meet the
 *  application requirements.  This class will only be generated as
 *  long as it does not already exist in the output directory.
 *
 * @package plugins.docman.lib
 */
class DocmanFilePeer extends BaseDocmanFilePeer {
	private static $_pkCache = array();

	/**
	 * Retrieve a single object by pkey or NULL if not found.
	 * This method was added by propel_customizer.php.
	 *
	 * @param	mixed $pk	The primary key
	 * @param	bool $useCache	Whether to use caching. This will increase performance if the same object
	 *							is accessed several times
	 * @param	Connection $con	The connection to use
	 * @return	DocmanFile
	 */
	public static function retrieveByPk($pk, $useCache = true, $con = null) {
		if ($useCache && array_key_exists($pk, DocmanFilePeer::$_pkCache)) {
			return DocmanFilePeer::$_pkCache[$pk];
		}
		
		$object = parent::retrieveByPk($pk, $con);
		if ($useCache) {
			DocmanFilePeer::$_pkCache[$pk] = $object;
		}
		return $object;
	}
	
	/**
	 * Removes an object from the cache. This is usually done after saving.
	 * This method was added by propel_customizer.php.
	 *
	 * @param	mixed $pk	The primary key of the object to remove
	 */
	public static function invalidate($pk) {
		if (array_key_exists($pk, DocmanFilePeer::$_pkCache)) {
			unset(DocmanFilePeer::$_pkCache[$pk]);
		}
	}

	
	/**
	 * Retrieves the docs with pending approval for a project
	 * Returns a Resultset
	 * @param int $project_id
	 */
	public static function doSelectPendingDocsRS($project_id) {
		
		$sql="SELECT df.docman_file_id,dfv.docman_file_version_id,dfv.version_number,dfv.is_approved,
		dfv.created_by, dfv.changes, dfv.create_date, fs.filesystem_id, fs.file_name_safe,df.description
		FROM docman_folder, docman_file df, docman_file_version dfv,
		filesystem fs
		WHERE 
		docman_folder.project_id='".intval($project_id)."'
		AND docman_folder.docman_folder_id=df.docman_folder_id
		AND df.docman_file_id=dfv.docman_file_id
		AND fs.section='docmanfileversion'
		AND fs.ref_id=dfv.docman_file_version_id
		AND dfv.is_approved=false";

		$con = Propel::getConnection(DocmanFolderPeer::DATABASE_NAME);
		$stmt = $con->createStatement();
		$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_ASSOC);
		
		return $rs;
	}
	
	
	/**
	 * Retrieves a list of projects that have docs pending of approval.
	 * If project_ids is null, it will fetch all projects in the DB
	 * Returns an array of project ids
	 * @param array $project_ids
	 */
	public static function doSelectProjectsWithPendingDocs($project_ids=null) {
		
		if(is_array($project_ids)) {
			$ids = implode(',',$project_ids);
			$sqlprj = "docman_folder.project_id IN ($ids)";
		} else {
			$sqlprj = '';
		}
		
		$sql="SELECT DISTINCT docman_folder.project_id
		FROM docman_folder, docman_file df, docman_file_version dfv, filesystem fs
		WHERE 
		$sqlprj
		AND docman_folder.docman_folder_id=df.docman_folder_id
		AND df.docman_file_id=dfv.docman_file_id
		AND fs.section='docmanfileversion'
		AND fs.ref_id=dfv.docman_file_version_id
		AND dfv.is_approved=false";

		$con = Propel::getConnection(DocmanFolderPeer::DATABASE_NAME);
		$stmt = $con->createStatement();
		$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_NUM);
		
		$ret = array();
		while($rs->next()) {
			$ret[] = $rs->getInt(1);
		}
		return $ret;
	}

	/**
	 * Retrieves a list of doc versions for this doc
	 */
	public static function doSelectDocumentVersionsRS ($docid, $approved=true) {

		$sql="SELECT dfv.docman_file_id, dfv.docman_file_version_id,dfv.version_number,dfv.is_approved,
		dfv.created_by, dfv.changes, dfv.create_date, fs.filesystem_id, fs.file_name_safe,df.description
		FROM docman_folder, docman_file df, docman_file_version dfv,
		filesystem fs
		WHERE df.docman_file_id = '".intval($docid)."'
		AND docman_folder.docman_folder_id=df.docman_folder_id
		AND df.docman_file_id=dfv.docman_file_id
		AND fs.section='docmanfileversion'
		AND fs.ref_id=dfv.docman_file_version_id";
		if ($approved) {
			$sql .= " AND dfv.is_approved=true";
		}
		$sql .= " ORDER BY dfv.version_number ASC";
		$con = Propel::getConnection(DocmanFolderPeer::DATABASE_NAME);
		$stmt = $con->createStatement();
		return $stmt->executeQuery($sql, ResultSet::FETCHMODE_ASSOC);

	}
}
?>
