<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2005-11-11
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
// The parent class
require_once 'plugins/forum/lib/om/BaseForumPeer.php';

// The object class
include_once 'plugins/forum/lib/Forum.php';

/** 
 * The skeleton for this class was autogenerated by Propel on:
 *
 * [Sun Nov 13 22:49:46 2005]
 *
 *  You should add additional methods to this class to meet the
 *  application requirements.  This class will only be generated as
 *  long as it does not already exist in the output directory.
 *
 * @package plugins.forum.lib 
 */
class ForumPeer extends BaseForumPeer {
	private static $_pkCache = array();

	/**
	 * Retrieve a single object by pkey or NULL if not found.
	 * This method was added by propel_customizer.php.
	 *
	 * @param	mixed $pk	The primary key
	 * @param	bool $useCache	Whether to use caching. This will increase performance if the same object
	 *							is accessed several times
	 * @param	Connection $con	The connection to use
	 * @return	Forum
	 */
	public static function retrieveByPk($pk, $useCache = true, $con = null) {
		if ($useCache && array_key_exists($pk, ForumPeer::$_pkCache)) {
			return ForumPeer::$_pkCache[$pk];
		}
		
		$object = parent::retrieveByPk($pk, $con);
		if ($useCache) {
			ForumPeer::$_pkCache[$pk] = $object;
		}
		return $object;
	}
	
	/**
	 * Removes an object from the cache. This is usually done after saving.
	 * This method was added by propel_customizer.php.
	 *
	 * @param	mixed $pk	The primary key of the object to remove
	 */
	public static function invalidate($pk) {
		if (array_key_exists($pk, ForumPeer::$_pkCache)) {
			unset(ForumPeer::$_pkCache[$pk]);
		}
	}
	
	/**
	 * Find the forums with the specified section name and ref_id
	 * @param	string	$section
	 * @param	int	$ref_id
	 * @return	Forum
	 */
	public static function retrieveForums($section, $ref_id) {
		$c = new Criteria();
		$c->add(self::SECTION, $section);
		$c->add(self::REF_ID, $ref_id);
		return self::doSelect($c);
	}
	
	public static function doSelectWithLatestMessage(Criteria $c) {
		$ret = array();
		$forums = self::doSelect($c);
		foreach ($forums as $forum) {
			$item = array();
			$item['forum'] = $forum;
			
			// Now get the latest message from this forum
			$c2 = new Criteria();
			$c2->add(ForumThreadPeer::FORUM_ID, $forum->getForumId());
			$c2->addJoin(ForumMessagePeer::FORUM_THREAD_ID, ForumThreadPeer::FORUM_THREAD_ID);
			$c2->addDescendingOrderByColumn(ForumMessagePeer::POST_DATE);
			$c2->setOffset(0);
			$c2->setLimit(1);
			$res = ForumMessagePeer::doSelectJoinUser($c2);
			if (count($res) == 0) {
				$item['latest_message'] = null;
			} else {
				$item['latest_message'] = $res[0];
			}
			
			$ret[] = $item;
		}
		
		return $ret;
	}
	
	/**
	 * Get the latest threads from the forums the user is monitoring
	 */
	public static function getLatestMonitoredThreads(User $user, $section, $limit=5) {
		// Get the forums in $section the user is monitoring
		$c = new Criteria();
		$c->add(MonitorPeer::SECTION, 'forum');
		$c->add(MonitorPeer::USER_ID, $user->getUserId());
		$c->add(self::SECTION, $section);
		$c->addJoin(MonitorPeer::REF_ID, self::FORUM_ID);
		$c->addJoin(ProjectPeer::PROJECT_ID, self::REF_ID);
		$c->addAscendingOrderByColumn(ProjectPeer::PROJECT_NAME);
		$c->addDescendingOrderByColumn(ForumPeer::MOST_RECENT_DATE);
		
		// Fakes a "doSelectJoinProject"
		self::addSelectColumns($c);
		ProjectPeer::addSelectColumns($c);
		$rs = self::doSelectRS($c);
		
		$result = array();
		$last_project = 0;
		$i = -1;
		while ($rs->next()) {
			$startcol = (self::$numColumns - self::$numLazyLoadColumns) + 1;
			$forum = new Forum();
			$forum->hydrate($rs);
			$project = new Project();
			$project->hydrate($rs, $startcol);
			
			if ($last_project != $project->getProjectId()) {
				$last_project = $project->getProjectId();
				$i++;
				$result[$i]['project'] = $project;
				$result[$i]['threads'] = array();
			}
			
			// Get the latest messages from this forum
			$latest_threads = $forum->getTopThreads($limit);
			if (count($latest_threads) == 0) {
				continue;
			}
			
			$result[$i]['threads'][] = array('forum' => $forum, 'latest_threads' => $latest_threads);
		}
		
		return $result;
	}

}
?>