<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2005-11-11
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once('plugins/frs/lib/om/BaseFrsPackage.php');
require_once("plugin/ActionManager.php");
require_once('exceptions/PermissionDeniedException.php');
require_once('GFPermission.interface.php');
require_once('plugin/ActionManager.php');
require_once('core/UserPeer.php');
require_once('core/MonitorPeer.php');

/** 
 * The skeleton for this class was autogenerated by Propel  on:
 *
 * [Sun Nov 13 22:49:46 2005]
 *
 * You should add additional methods to this class to meet the
 * application requirements.  This class will only be generated as
 * long as it does not already exist in the output directory.
 *
 * @package plugins.frs.lib 
 */
class FrsPackage extends BaseFrsPackage implements GFWidget, GFPermission {

	const SECTION = 'frspackage';
	const STATUS_ACTIVE=1;
	const STATUS_HIDDEN=2;
	const PERMLEVEL_NOACCESS=0;
	const PERMLEVEL_READ=1;
	const PERMLEVEL_ADMIN=2;

	var $monitoring_users; // the users that are monitoring this package

	/**
	 * isPublic - true/false
	 * @return boolean
	 */
	function isPublic() {
		return $this->getIsPublic();
	}

	/**
	 * Specifies if the current user can create an entry in the database for this object
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanCreate() {
		if ($this->userIsAdmin()) {
			return true;
		}
		return false;
	}

	/**
	 * If this person is an admin for this object
	 * @return boolean
	 */
	public function userIsAdmin() {
		//requires manual customization
		$perm = $this->getProject()->getPermission(); 
		//user has permission to write
		if ($perm->check(FrsPlugin::SECTION, 0, FrsPlugin::WRITE) || $perm->check(self::SECTION, $this->getFrsPackageId(), FrsPackage::PERMLEVEL_ADMIN)) { 
			return true;
		}		
	}

	/**
	 * Specifies if the current user can update this object's entry in the database
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanUpdate() {
		return $this->userIsAdmin();
	}
	
	/**
	 * Specifies if the current user can delete this object's entry from the database
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanDelete() {
		return $this->userIsAdmin();
	}

	/**
	 * Specifies if the current user has permissions to access this object
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanRead() {
		if ($this->isPublic()) {
			return true;
		}
		if ($this->userIsAdmin()) {
			return true;
		}
		//COMMUNITY EXPRESS
		if (Session::getSession()->isLoggedIn() && Session::getSession()->getUser()->isProjectMember($this->getProject())) {
			return true;
		}
/*
		$perm = $this->getProject()->getPermission();
		if ( $perm->check(self::SECTION, $this->getFrsPackageId() , FrsPackage::PERMLEVEL_READ) ) { // read permission
			return true;
		}
*/
		return false;
	}


	/**
	 * Removes this object from datastore and sets delete attribute. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 * 
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 * @see FrsPackage::setDeleted()
	 * @see FrsPackage::isDeleted()
	 */
	function delete($con = null) {
		if (!$this->userCanDelete()) {
			throw new PermissionDeniedException("Could not delete FrsPackage object: permission denied");
		}
		//delete monitors
		$crit=new Criteria();
		$crit->add(MonitorPeer::SECTION,self::SECTION);
		$crit->add(MonitorPeer::REF_ID,$this->getFrsPackageId());
		MonitorPeer::doDelete($crit);
		
		// Audit trail
		$audit = new AuditTrail();
		$audit->setUser(Session::getSession()->getUser());
		$audit->setSection(Project::SECTION);
		$audit->setRefId($this->getProjectId());
		$audit->setFieldName(self::SECTION);
		$audit->setNewValue('_deleted_');
		$audit->setOldValue($this->getPackageName().' '.Language::getSessText('Frs.AuditDeleteAllContents'));
		$audit->save();


		ActionManager::notify("FRS_PACKAGE_DELETE", $this);
		parent::delete($con);		
	}

	/**
	 * Stores the object in the database using the Propel methods. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 *
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 */
	function save($con = null) {
		if ($this->isNew() && !$this->userCanCreate()) {
			throw new PermissionDeniedException("Could not create FrsPackage object: permission denied");
		} else if (!$this->userCanUpdate()) {
			throw new PermissionDeniedException("Could not update FrsPackage object: permission denied");
		}
		$errors=array();

		if (strlen($this->package_name) < 4) {
			$errors['package_name'] = Language::getSessText('FrsPackage.EmptyPackageName');
		} else {
			// Check for duplicated package name
			$c = new Criteria();
			$c->add(FrsPackagePeer::PROJECT_ID, $this->getProjectId());
			$c->add(FrsPackagePeer::PACKAGE_NAME, $this->package_name, Criteria::EQUAL);
			if (!$this->isNew()) {
				$c->add(FrsPackagePeer::FRS_PACKAGE_ID, $this->frs_package_id, Criteria::NOT_EQUAL);
			}
			$match_name = FrsPackagePeer::doSelect($c);
			if (count($match_name) > 0) {
				$errors['package_name'] = Language::getSessText('FrsPackage.DuplicatedName');
			}
		}
		if (count($errors) > 0) {
			throw new InvalidDataException($errors);
		}
		
		$is_new = $this->isNew();
		$createrolesettings = $is_new;

		parent::save($con);
		ActionManager::notify("FRS_PACKAGE_SAVE", $this);
	
		if ($createrolesettings) {
			$con = Propel::getConnection(TrackerPeer::DATABASE_NAME);
			$sql="INSERT INTO role_setting (role_id,section,ref_id,value) 
				SELECT role_id,'".self::SECTION."','".$this->getPrimaryKey()."',
				'".self::PERMLEVEL_READ."' 
				FROM role WHERE project_id='".$this->getProject()->getProjectId()."'";
			$stmt = $con->createStatement();
			$rs = $stmt->executeQuery($sql);
		}
	}

	/**
	 * Returns an array of user objects corresponding to the ones that are monitoring this package or project
	 *
	 * @return array	Users
	 */
	function getMonitoringUsers() {
		if (!$this->monitoring_users) {
			$this->monitoring_users = MonitorPeer::getMonitoringUsers(self::SECTION, $this->getPrimaryKey());
		}
		return $this->monitoring_users;
	}
	
	/**
	 * Tells if the user is monitoring this package
	 * @return bool
	 */
	public function userIsMonitoring(User $user) {
		$c = new Criteria();
		$c->add(MonitorPeer::SECTION, self::SECTION);
		$c->add(MonitorPeer::REF_ID, $this->getPrimaryKey());
		$c->add(MonitorPeer::USER_ID, $user->getUserId());
		$mon = MonitorPeer::doSelectOne($c);
		return (!is_null($mon)) ? true : false;
	}

	
	function setPackageName($filename) {
		$filename = StringUtils::removeInvalidChars($filename);
		parent::setPackageName($filename);
	}
	
	// These functions come from GFWidget
	public function getID() {
		return $this->getFrsPackageId();
	}
	public function getName() {
		return $this->getPackageName();
	}
	
	
}
?>
