<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('plugins/tracker/lib/Tracker.php');
require_once('plugins/tracker/lib/TrackerItem.php');
require_once('plugins/tracker/lib/TrackerExtraField.php');
require_once('plugins/tracker/lib/TrackerExtraFieldData.php');
require_once('lib/html/HTMLWidget.php');
require_once('plugins/frs/lib/FrsReleasePeer.php');

class TrackerExtraFieldRenderer {

	private $Tracker;
	function __construct(Tracker $Tracker) {
		$this->Tracker = $Tracker;
	}

	function getQuerySelected($criteria,$TrackerItemQuery) {
		$fields = $this->Tracker->getTrackerExtraFields($criteria);
		$selected=$TrackerItemQuery->getExtraFields();
		$selected['PRIORITY']=$TrackerItemQuery->getPriority();
		if (!$selected['PRIORITY']) {
			$selected['PRIORITY']=array();
		}
		$selected['ASSIGNEDTO']=$TrackerItemQuery->getAssignedTo();
		if (!$selected['ASSIGNEDTO']) {
			$selected['ASSIGNEDTO']=array();
		}
		foreach ($fields as $field) {
			if (!isset($selected[$field->getTrackerExtraFieldId()][0])) {
				$selected[$field->getTrackerExtraFieldId()][0]='';
			}
		}
		return $selected;
	}

	function getDefaultSelected($criteria) {
		$fields = $this->Tracker->getTrackerExtraFields($criteria);
		$selected=array();
		foreach ($fields as $field) {
			//
			//	may not have an element selected for default
			//
			$selected[$field->getTrackerExtraFieldId()][]='';
		}
		$selected['PRIORITY']='';
		$selected['ASSIGNEDTO']=array(0);
		return $selected;
	}

	function getTrackerItemSelected(TrackerItem $TrackerItem, Criteria $criteria) {
		$TrackerExtraFieldData = $TrackerItem->getTrackerExtraFieldDatas();
		$selected = array();
		foreach ($TrackerExtraFieldData as $tefd) {
			$selected[$tefd->getTrackerExtraFieldId()][]=$tefd->getFieldData();
		}
		$fields = $this->Tracker->getTrackerExtraFields($criteria);
		foreach ($fields as $field) {
			//
			//	may not have an element selected for default
			//
			if (!isset($selected[$field->getTrackerExtraFieldId()][0])) {
				switch ($field->getFieldType()) {
					case TrackerExtraField::TYPE_TEXTAREA:
					case TrackerExtraField::TYPE_DATE:
					case TrackerExtraField::TYPE_NUMERIC:
					case TrackerExtraField::TYPE_SERIAL:
					case TrackerExtraField::TYPE_TEXT: {
						$selected[$field->getTrackerExtraFieldId()][]='';
						break;
					}
					default: {
						$selected[$field->getTrackerExtraFieldId()][]=100;
					}
				}
			}
		}
		$selected['PRIORITY']=$TrackerItem->getPriority();
		$assignees = $TrackerItem->getTrackerItemAssignees();
		$assign=array();
		foreach ($assignees as $assignee) {
			$assign[]=$assignee->getAssignee();
		}
		$selected['ASSIGNEDTO']=$assign;
		return $selected;
	}

	function renderFieldsReadOnly($TrackerItem) {
		$criteria = new Criteria();
		$selected = $this->getTrackerItemSelected($TrackerItem,$criteria);
		$template = $this->generateTemplate($criteria);
		$fields = $this->Tracker->getTrackerExtraFields($criteria);

		//assignee
		$crit = new Criteria();
		$crit->add(UserPeer::USER_ID,$selected['ASSIGNEDTO'],Criteria::IN);
		$assignees = UserPeer::doSelect($crit);
		$str='';
		foreach ($assignees as $assignee) {
			$str .= $assignee->getUserLink().'<br />';
		}
		$template = str_replace('<!--EFASSIGNEDTO-->',$str,$template);

		//priority
		$priorities=$this->Tracker->getPriorities();
		$str=$priorities[$selected['PRIORITY']];
		$template = str_replace('<!--EFPRIORITY-->',$str,$template);

		//
		//	Custom fields
		//
		foreach ($fields as $field) {
			$sel = $selected[$field->getTrackerExtraFieldId()];
			switch ($field->getFieldType()) {
			case TrackerExtraField::TYPE_SELECT:
			case TrackerExtraField::TYPE_RADIO:
			case TrackerExtraField::TYPE_STATUS:
			case TrackerExtraField::TYPE_MEMBERS:
			case TrackerExtraField::TYPE_RELEASE: {
//MONO HARD CODED TIE TO FRS
				if (isset($selected[$field->getTrackerExtraFieldId()])) {
					if ($field->getFieldType() == TrackerExtraField::TYPE_RELEASE) {
						$element=FrsReleasePeer::retrieveByPk($selected[$field->getTrackerExtraFieldId()][0]);
						if (is_object($element)) {
							$str=$element->getReleaseName();
						} else {
							$str='N/A';
						}
					} else if ($field->getFieldType() == TrackerExtraField::TYPE_MEMBERS){
						$element=UserPeer::retrieveByPk($selected[$field->getTrackerExtraFieldId()][0]);
						if(is_object($element)) {
							$str=$element->getUserLink() . '<br />';
						} else {
							$str='N/A';
						}
					} else {
						$element=TrackerExtraFieldElementPeer::retrieveByPk($selected[$field->getTrackerExtraFieldId()][0]);
						if (is_object($element)) {
							$str=$element->getElementName();
						} else {
							$str='N/A';
						}
					}
				} else {
					$str='N/A';
				}
				break;
			}
			case TrackerExtraField::TYPE_CHECKBOX:
			case TrackerExtraField::TYPE_MULTISELECT: {

				$crit = new Criteria();
				$crit->add(TrackerExtraFieldElementPeer::ELEMENT_ID,$selected[$field->getTrackerExtraFieldId()],Criteria::IN);
				$elements=TrackerExtraFieldElementPeer::doSelect($crit);
				$str='';
				foreach ($elements as $element) {
					$str .= $element->getElementName().'<br />';
				}
				break;
			}
			case TrackerExtraField::TYPE_TEXT:
			case TrackerExtraField::TYPE_DATE:
			case TrackerExtraField::TYPE_NUMERIC:
			case TrackerExtraField::TYPE_SERIAL:
			case TrackerExtraField::TYPE_TEXTAREA: {

				$str=$selected[$field->getTrackerExtraFieldId()][0];

				break;
			}
			}//end switch
			$template = str_replace('<!--EF'.$field->getTrackerExtraFieldId().'-->',$str,$template);
			unset($str);
			unset($sel);
		}
		return $template;
	}

	function renderFields($mode,$TrackerItem=null,$TrackerItemQuery=null,$user_selected=array()) {
		switch ($mode) {
			case 'QUERY': {
				$criteria = new Criteria();
				$criteria->add(TrackerExtraFieldPeer::FIELD_TYPE,explode(',',TrackerExtraField::QUERY_TYPES),Criteria::IN);
				$criteria->addAscendingOrderByColumn(TrackerExtraFieldPeer::FIELD_ORDER);
				$defaultSelected = $this->getQuerySelected($criteria,$TrackerItemQuery);
				break;
			}
			case 'SUBMIT': {
				$criteria = new Criteria();
				$criteria->add(TrackerExtraFieldPeer::SHOW_ON_SUBMIT,'true');
				$criteria->addAscendingOrderByColumn(TrackerExtraFieldPeer::FIELD_ORDER);
				$defaultSelected = $this->Tracker->getTrackerDefaultSelected($criteria);
				break;
			}
			case 'EDIT': {
				$criteria = new Criteria();
				$criteria->addAscendingOrderByColumn(TrackerExtraFieldPeer::FIELD_ORDER);
				$defaultSelected = $this->getTrackerItemSelected($TrackerItem,$criteria);
				break;
			}
			case 'MASSUPDATE': {
				$criteria = new Criteria();
				$criteria->add(TrackerExtraFieldPeer::FIELD_TYPE,explode(',',TrackerExtraField::INT_TYPES),Criteria::IN);
				$criteria->addAscendingOrderByColumn(TrackerExtraFieldPeer::FIELD_ORDER);
				$defaultSelected = $this->getDefaultSelected($criteria);
				break;
			}
			case 'QUICKBROWSE': {
				$criteria = new Criteria();
				$criteria->add(TrackerExtraFieldPeer::USE_IN_QUICK_BROWSE,'true');
				$criteria->add(TrackerExtraFieldPeer::FIELD_TYPE,explode(',',TrackerExtraField::INT_TYPES),Criteria::IN);
				$criteria->addAscendingOrderByColumn(TrackerExtraFieldPeer::FIELD_ORDER);
				$defaultSelected = $this->getQuerySelected($criteria,$TrackerItemQuery);
				break;
			}
		}
		
		// Merge the selected values with the selected by default
		// Note that $user_selected comes from $_POST so it has a very different format than
		// $defaultSelected
		$selected = $defaultSelected;
		
		if (isset($user_selected['priority'])) {
			$selected['PRIORITY'] = $user_selected['priority'];
		}
		if (isset($user_selected['assigned_to'])) {
			$selected['ASSIGNEDTO'] = $user_selected['assigned_to'];
		}
		
		if (array_key_exists('extra_field', $user_selected) && is_array($user_selected['extra_field'])) {
			// Extra field data is set
			foreach ($user_selected['extra_field'] as $extra_field_id => $value) {
				if (is_array($value)) {
					// for multiselect fields
					$selected[$extra_field_id] = $value;
				} else {
					$selected[$extra_field_id][0] = $value;
				}
			}
		}
		
		
		
		$template = $this->generateTemplate($criteria,$mode);

		//kind of getting hacky, but we want to re-use the template generator...
		if ($mode == 'QUICKBROWSE') {
			//strip the <tr> tags - everything should be on one row
			$template = str_replace('<tr>','',$template);
			$template = str_replace('</tr>','',$template);
		}
		$fields = $this->Tracker->getTrackerExtraFields($criteria);

		//
		//	Assignee
		//
		if ($mode == 'QUICKBROWSE') {
			$str = HTMLWidget::select('assigned_to',$this->Tracker->getTechnicians(),$selected['ASSIGNEDTO'],HTMLWidget::OPTION_ANY,true,null,true,null);
		} elseif ($mode == 'MASSUPDATE') {
			$str = HTMLWidget::select('assigned_to',$this->Tracker->getTechnicians(),'',HTMLWidget::OPTION_ANY,true,null,true,'No Change');
		} elseif ($mode == 'SUBMIT') {
			if ($this->Tracker->userIsAdmin()) {
				$str = HTMLWidget::multiselect('assigned_to[]',$this->Tracker->getTechnicians(),$selected['ASSIGNEDTO']);
			} else {
				$str = '(auto)';
			}
		} else {
			$str = HTMLWidget::multiselect('assigned_to[]',$this->Tracker->getTechnicians(),$selected['ASSIGNEDTO'],100,true,null,false,null,5);
		}
		$template = str_replace('<!--EFASSIGNEDTO-->',$str,$template);

		//
		//	priority
		//
		if ($mode == 'QUERY') {
			$str = HTMLWidget::multiSelectFromAssoc('priority[]',$this->Tracker->getPriorities(),$selected['PRIORITY'],100,false);
		} elseif ($mode == 'QUICKBROWSE') {
			$str = HTMLWidget::selectBoxFromAssoc($this->Tracker->getPriorities(),'priority',$selected['PRIORITY'],false,false,null,true,null);
		} elseif ($mode == 'MASSUPDATE') {
			$str = HTMLWidget::selectBoxFromAssoc($this->Tracker->getPriorities(),'priority',HTMLWidget::OPTION_ANY,false,false,null,true,'No Change');
		} else {
			$str = HTMLWidget::selectBoxFromAssoc($this->Tracker->getPriorities(),'priority',$selected['PRIORITY'],false,false);
		}
		$template = str_replace('<!--EFPRIORITY-->',$str,$template);


		//
		//	Custom fields
		//
		foreach ($fields as $field) {

			if ($mode == 'QUERY') {
				//queries use multi-select boxes for everything EXCEPT date and numeric
				if ($field->getFieldType() == TrackerExtraField::TYPE_STATUS) {
					$show_none=false;
				} else {
					$show_none=true;
				}
				switch ($field->getFieldType()) {
//date. numeric and serial fields have a 'range'
					case TrackerExtraField::TYPE_DATE:{
						$str = '<input type="text" id="extra_field'.$field->getTrackerExtraFieldId().'_0" name="extra_field['.$field->getTrackerExtraFieldId().'][0]" value="'.(isset($selected[$field->getTrackerExtraFieldId()][0]) ? $selected[$field->getTrackerExtraFieldId()][0] : '').'" size="11" maxlength="11"/><a href="javascript:NewCal(\'extra_field'.$field->getTrackerExtraFieldId().'_0\',\'MMMddyyyy\')"><img src="/themes/gforge5/images/cal.gif" alt="Pick a date" border="0" height="16" width="16"></a>&nbsp;<input type="text" id="extra_field'.$field->getTrackerExtraFieldId().'_1" name="extra_field['.$field->getTrackerExtraFieldId().'][1]" value="'.(isset($selected[$field->getTrackerExtraFieldId()][1]) ? $selected[$field->getTrackerExtraFieldId()][1] : '').'" size="11" maxlength="11"/><a href="javascript:NewCal(\'extra_field'.$field->getTrackerExtraFieldId().'_1\',\'MMMddyyyy\')"><img src="/themes/gforge5/images/cal.gif" alt="Pick a date" border="0" height="16" width="16"></a>';
						break;
					}
					case TrackerExtraField::TYPE_SERIAL:{
						$str = '<input type="text" name="extra_field['.$field->getTrackerExtraFieldId().'][0]" value="'. (isset($selected[$field->getTrackerExtraFieldId()][0]) ? $selected[$field->getTrackerExtraFieldId()][0] : '').'" size="5"/>&nbsp;<input type="text" name="extra_field['.$field->getTrackerExtraFieldId().'][1]" value="'.(isset($selected[$field->getTrackerExtraFieldId()][1]) ? $selected[$field->getTrackerExtraFieldId()][1] : '').'" size="5" />';
					  	break; 
					 }
					case TrackerExtraField::TYPE_NUMERIC:{
						$str = '<input type="text" name="extra_field['.$field->getTrackerExtraFieldId().'][0]" value="'. (isset($selected[$field->getTrackerExtraFieldId()][0]) ? $selected[$field->getTrackerExtraFieldId()][0] : '').'" size="'.$field->getAttribute1().'" maxlength="'.$field->getAttribute2().'"/>&nbsp;<input type="text" name="extra_field['.$field->getTrackerExtraFieldId().'][1]" value="'.(isset($selected[$field->getTrackerExtraFieldId()][1]) ? $selected[$field->getTrackerExtraFieldId()][1] : '').'" size="'.$field->getAttribute1().'" maxlength="'.$field->getAttribute2().'"/>';
						break;
					}
					default: {
						$str = HTMLWidget::multiselect('extra_field['.$field->getTrackerExtraFieldId().'][]',$field->getTrackerExtraFieldElements(),$selected[$field->getTrackerExtraFieldId()],null,$show_none);
					}
				}
			} elseif ($mode == 'MASSUPDATE') {
				if ($field->getFieldType() == TrackerExtraField::TYPE_STATUS) {
					$show_none=false;
				} else {
					$show_none=true;
				}
				$str = HTMLWidget::select('extra_field['.$field->getTrackerExtraFieldId().']',$field->getTrackerExtraFieldElements(),'',HTMLWidget::OPTION_ANY,$show_none,null,true,'No Change');
			} elseif ($mode == 'QUICKBROWSE') {
				//mass update and browse always use select boxes
				if ($field->getFieldType() == TrackerExtraField::TYPE_STATUS) {
					$show_none=false;
				} else {
					$show_none=true;
				}
				$str = HTMLWidget::select('extra_field['.$field->getTrackerExtraFieldId().']',$field->getTrackerExtraFieldElements(),$selected[$field->getTrackerExtraFieldId()],HTMLWidget::OPTION_ANY,$show_none,null,true,null);
			} elseif ($field->getFieldType() == TrackerExtraField::TYPE_SELECT) {
				if ($mode == 'EDIT') {
					// Only show the available items defined by the workflow
					// Note that we use $defaultSelected instead of $selected. This must be done
					// in case the user selects the "none" value.
					$selected_value = $defaultSelected[$field->getTrackerExtraFieldId()][0];
					$available_values = $field->getTrackerExtraFieldElementsWorkflow($selected_value);
					
					if ($field->getUseWorkflow()) {
						// Show the "none" value only if it is the current value
						$show_none = ($selected_value == TrackerExtraFieldElement::ELEMENT_NONE);
					} else {
						$show_none = true;
					}
					
					$str = HTMLWidget::select('extra_field['.$field->getTrackerExtraFieldId().']',$available_values,$selected_value,null,$show_none);
				} elseif ($mode == 'SUBMIT') {
					// Show the "none" value only if the field doesn't use workflow
					$show_none = !$field->getUseWorkflow();
					$str = HTMLWidget::select('extra_field['.$field->getTrackerExtraFieldId().']',$field->getTrackerExtraFieldElements(),$selected[$field->getTrackerExtraFieldId()][0],null,$show_none);
				} else {
					$str = HTMLWidget::select('extra_field['.$field->getTrackerExtraFieldId().']',$field->getTrackerExtraFieldElements(),$selected[$field->getTrackerExtraFieldId()][0]);
				}
			} elseif ($field->getFieldType() == TrackerExtraField::TYPE_RELEASE) {
				$str = HTMLWidget::select('extra_field['.$field->getTrackerExtraFieldId().']',$field->getTrackerExtraFieldElements(),$selected[$field->getTrackerExtraFieldId()][0]);
			} elseif ($field->getFieldType() == TrackerExtraField::TYPE_CHECKBOX) {
				$str = HTMLWidget::checkbox('extra_field['.$field->getTrackerExtraFieldId().'][]',$field->getTrackerExtraFieldElements(),$selected[$field->getTrackerExtraFieldId()]);
			} elseif ($field->getFieldType() == TrackerExtraField::TYPE_RADIO) {
				if ($mode == 'EDIT') {
					// Only show the available items defined by the workflow
					$selected_value = $defaultSelected[$field->getTrackerExtraFieldId()][0];
					$available_values = $field->getTrackerExtraFieldElementsWorkflow($selected_value);

					if ($field->getUseWorkflow()) {
						// Show the "none" value only if it is the current value
						$show_none = ($selected_value == TrackerExtraFieldElement::ELEMENT_NONE);
					} else {
						$show_none = true;
					}

					$str = HTMLWidget::radio('extra_field['.$field->getTrackerExtraFieldId().']',$available_values,$selected_value,null,$show_none);
				} elseif ($mode == 'SUBMIT') {
					// Show the "none" value only if the field doesn't use workflow
					$show_none = !$field->getUseWorkflow();
					$str = HTMLWidget::radio('extra_field['.$field->getTrackerExtraFieldId().']',$field->getTrackerExtraFieldElements(),$selected[$field->getTrackerExtraFieldId()][0],null,$show_none);
				} else {
					$str = HTMLWidget::radio('extra_field['.$field->getTrackerExtraFieldId().']',$field->getTrackerExtraFieldElements(),$selected[$field->getTrackerExtraFieldId()][0]);
				}
			} elseif ($field->getFieldType() == TrackerExtraField::TYPE_MULTISELECT
			|| $field->getFieldType() == TrackerExtraField::TYPE_MEMBERS) { 
				$str = HTMLWidget::multiselect('extra_field['.$field->getTrackerExtraFieldId().'][]',$field->getTrackerExtraFieldElements(),$selected[$field->getTrackerExtraFieldId()]);
			} elseif ($field->getFieldType() == TrackerExtraField::TYPE_TEXT) {
//size=attr1
//maxlength=attr2
				$str = '<input type="text" name="extra_field['.$field->getTrackerExtraFieldId().']" value="'.$selected[$field->getTrackerExtraFieldId()][0].'" size="'.$field->getAttribute1().'" maxlength="'.$field->getAttribute2().'"/>';
			} elseif ($field->getFieldType() == TrackerExtraField::TYPE_TEXTAREA) {
//rows = attr1
//cols=attr2
				$str = '<textarea name="extra_field['.$field->getTrackerExtraFieldId().']" cols="'.$field->getAttribute2().'" rows="'.$field->getAttribute1().'"/>'.$selected[$field->getTrackerExtraFieldId()][0].'</textarea>';
			} elseif ($field->getFieldType() == TrackerExtraField::TYPE_STATUS) {

				if ($mode == 'EDIT') {
					// Only show the available items defined by the workflow
					$selected_value = $defaultSelected[$field->getTrackerExtraFieldId()][0];
					$available_values = $field->getTrackerExtraFieldElementsWorkflow($selected_value);
					$str = HTMLWidget::select('extra_field['.$field->getTrackerExtraFieldId().']',$available_values,$selected_value,null,false);
				} else {
					$str = HTMLWidget::select('extra_field['.$field->getTrackerExtraFieldId().']',$field->getTrackerExtraFieldElements(),$selected[$field->getTrackerExtraFieldId()][0],null,false);
				}
				//$str = HTMLWidget::select('extra_field['.$field->getTrackerExtraFieldId().']',$field->getTrackerExtraFieldElements(),$selected[$field->getTrackerExtraFieldId()][0],null,false);

			} elseif ($field->getFieldType() == TrackerExtraField::TYPE_DATE) {

				$str = '<input type="text" id="extra_field'.$field->getTrackerExtraFieldId().'" name="extra_field['.$field->getTrackerExtraFieldId().']" value="'.$selected[$field->getTrackerExtraFieldId()][0].'" size="11" maxlength="11"/><a href="javascript:NewCal(\'extra_field'.$field->getTrackerExtraFieldId().'\',\'MMMddyyyy\')"><img src="/themes/gforge5/images/cal.gif" alt="Pick a date" border="0" height="16" width="16"></a>';

			} elseif ($field->getFieldType() == TrackerExtraField::TYPE_NUMERIC) {

//size=attr
//maxlength=attr2
				$field_size = ($field->getAttribute1() > 0) ? $field->getAttribute1() : 5;
				$field_maxlength = ($field->getAttribute2() > 0) ? $field->getAttribute2() : 5;
				$str = '<input type="text" name="extra_field['.$field->getTrackerExtraFieldId().']" value="'.$selected[$field->getTrackerExtraFieldId()][0].'" size="'.$field_size.'" maxlength="'.$field_maxlength.'"/>';
			} elseif ($field->getFieldType() == TrackerExtraField::TYPE_MEMBERS){
				$str = HTMLWidget::multiselect('extra_field['.$field->getTrackerExtraFieldId().'][]',$field->getTrackerExtraFieldElements(),$selected[$field->getTrackerExtraFieldId()]);
			} elseif ($field->getFieldType() == TrackerExtraField::TYPE_SERIAL){
				// The serial field shows up as read-only
				$str = $selected[$field->getTrackerExtraFieldId()][0];
			}
			$template = str_replace('<!--EF'.$field->getTrackerExtraFieldId().'-->',$str,$template);		
		//	unset($sel);
			unset($str);
		}
		return $template;
	}

	function generateTemplate($criteria=null, $mode=false) {

		$Lang = Session::getSession()->getLanguage();
		$return = '
			<!-- Start Extra Fields Rendering -->
			<tr>
					<td width="50%" valign="top"><strong>'.$Lang->getText('TrackerItem.PriorityName').':</strong><br /><!--EFPRIORITY--></td>
					<td width="50%" valign="top"><strong>'.$Lang->getText('TrackerItem.AssignedTo').':</strong><br /><!--EFASSIGNEDTO--></td>
			</tr>
			<tr>';

		$criteria->addAscendingOrderByColumn(TrackerExtraFieldPeer::FIELD_ORDER);
		$fields = $this->Tracker->getTrackerExtraFields($criteria);
		$count=count($fields);
		$col_count=0;
		$k=0;

		foreach ($fields as $field) {
			if ($mode == 'SUBMIT' || $mode == 'EDIT') {
				if ($mode == 'EDIT' && $field->getUseWorkflow()) {
					// Field is required because it has workflow rules
					$req='<span class="workflow_field">*</span>';
				} elseif ($field->getIsRequired()) {
					$req='<span class="required">*</span>';
				} else {
					$req='';
				}
			} else {
				$req='';
			}

			switch ($field->getFieldType()) {
				case TrackerExtraField::TYPE_MEMBERS:
				case TrackerExtraField::TYPE_SELECT:
				case TrackerExtraField::TYPE_MULTISELECT:
				case TrackerExtraField::TYPE_STATUS:
				case TrackerExtraField::TYPE_DATE: {
					//in query mode, the date is two fields entered as a range
					if ($mode == 'QUERY' && $field->getFieldType()==TrackerExtraField::TYPE_DATE) {
						$req='&nbsp;(Between)';
					}
				}
				case TrackerExtraField::TYPE_CHECKBOX:
				case TrackerExtraField::TYPE_RADIO:
				case TrackerExtraField::TYPE_RELEASE: {

					$return .= '
						<td width="50%" valign="top"><strong>'.$field->getFieldName().'</strong>:'.$req.'<br /><!--EF'.$field->getTrackerExtraFieldId().'--></td>';
					break;

				}
				case TrackerExtraField::TYPE_NUMERIC: {
					//in query mode, the number is two fields entered as a range
					if ($mode == 'QUERY' && $field->getFieldType()==TrackerExtraField::TYPE_NUMERIC) {
						$req='&nbsp;(Between)';
					}
				}
				case TrackerExtraField::TYPE_SERIAL: {
					//in query mode, the serial is two fields entered as a range
					if ($mode == 'QUERY' && $field->getFieldType() == TrackerExtraField::TYPE_SERIAL){
						$req='&nbsp;(Between)';
					}
				}
				case TrackerExtraField::TYPE_TEXT: {

					//text fields might be really wide, so need a row to themselves.
					if($field->getAttribute1() > 30){
						$colspan = 2;
						if($col_count == 1){
							$return .= '
							<td>&nbsp;</td>
							</tr>
							<tr>';
						}
					} else {
						$colspan=1;
					}

					$return .= '
					<td width="'.(50*$colspan).'%" colspan="'.$colspan.'" valign="top"><strong>'.$field->getFieldName().'</strong>:'.$req.'<br /><!--EF'.$field->getTrackerExtraFieldId().'--></td>';
					if($colspan == 2){
						//this is an ugly hack, we are reseting the column
						//counter to avoid problems with following fields
						$col_count = 1;
						$return .=' </tr>';
					}

					break;

				}
				case TrackerExtraField::TYPE_TEXTAREA: {

					//text areas might be really wide, so need a row to themselves.
					if($field->getAttribute2() > 30){
						$colspan = 2;
						//is this the second column?
						if($col_count == 1){
							$return .= '
							<td>&nbsp;</td>
							</tr>
							<tr>';
						}

					} else {
						$colspan = 1;
					}
										$return .= '
					<td width="'.(50*$colspan).'%" colspan="'.$colspan.'" valign="top"><strong>'.$field->getFieldName().'</strong>:'.$req.'<br /><!--EF'.$field->getTrackerExtraFieldId().'--></td>';
					if($colspan == 2){
						//this is an ugly hack, we are reseting the column
						//counter to avoid problems with following fields
						$col_count = 1;
						$return .=' </tr>';
					}
					
					break;

				}
			}//end switch
			$col_count++;
			//we've done two columns - if there are more to do, start a new row
			if (($col_count == 2) && ($k != $count-1)) {
				$col_count = 0;
				$return .= '
			</tr>
			<tr>';
			}
			$k++;
		}
		if ($col_count == 1) {
			$return .= '
					<td>&nbsp;</td>';
		}
		$return .= '
			</tr>
			<!-- End Extra Fields Rendering -->';
		return $return;

	}

}

?>
