<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
//
// This code edits a Tracker and saves it to the 
// database
//
require_once('plugins/tracker/lib/Tracker.php');
require_once('Validator.php');
require_once('exceptions/PermissionDeniedException.php');
require_once('core/Language.php');

if ($_SERVER['REQUEST_METHOD'] != 'POST') {
	throw new PermissionDeniedException();
}

$Tracker = TrackerPeer::retrieveByPk($_REQUEST['tracker_id']);
if (!is_object($Tracker) || !$Tracker->userIsAdmin()) {
	throw new PermissionDeniedException();
}

$errors = array();
$language = Language::getSessLang();

//
// TODO Validate the values passed by $_POST. Store the errors in $error like
// $errors["field_name"] = error_description
//

if (count($errors) == 0) {
	$Tracker->setTrackerName( Validator::sanitizeString($_POST['tracker_name']) );
	$Tracker->setDescription( Validator::sanitizeString($_POST['description']) );
	$Tracker->setIsPublic( isset($_POST['is_public']) ? true : false );
	$Tracker->setRestrictBrowse( isset($_POST['restrict_browse']) ? true : false );
	$Tracker->setEmailAllUpdates( isset($_POST['email_all_updates']) ? true : false );
	$Tracker->setEmailAddress( Validator::sanitizeString($_POST['email_address']) );
	$duedays=isset($_POST['due_period']) ? Validator::sanitizeInt($_POST['due_period']) : 15;
	$Tracker->setDuePeriod( $duedays*60*60*24 );
	$Tracker->setSubmitInstructions( Validator::sanitizeString($_POST['submit_instructions']) );
	$Tracker->setBrowseInstructions( Validator::sanitizeString($_POST['browse_instructions']) );
	
	$Tracker->setShowDependenciesTab( isset($_POST['show_dependencies_tab']) ? true : false );
	$Tracker->setShowDuplicatesTab( isset($_POST['show_duplicates_tab']) ? true : false );
	$Tracker->setShowAssociationsTab( isset($_POST['show_associations_tab']) ? true : false );
	$Tracker->setShowTimeTrackingTab( isset($_POST['show_time_tracking_tab']) ? true : false );
	$Tracker->setShowTagsTab( isset($_POST['show_tags_tab']) ? true : false );
	
	try {
		$Tracker->save();
		
		if (isset($_REQUEST['redirect_to'])) {
			$redirect_to = $_REQUEST['redirect_to'];
		} else {
			$redirect_to = GFUrl::getUrl()->currentBaseURL('TrackerBrowse');
		}
		
		GFUrl::getUrl()->redirect($redirect_to, array('feedback' => $language->getText('Tracker.EditFeedback')));
	} catch (InvalidDataException $ex) {
		$errors = array_merge($errors, $ex->getErrors());
		$feedback = '* '.implode('<br />* ', $errors);
		include('TrackerEdit.php');
	}
} else {
	$feedback = implode('<br />', $errors);
	include('TrackerEdit.php');
}

?>
