<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id$
 * @author Julian Yerfino	jyerfino@gforgegroup.com
 * @date 
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once ("lib/external/jpgraph/jpgraph.php");
require_once ("lib/external/jpgraph/jpgraph_line.php");
require_once ("lib/external/jpgraph/jpgraph_canvas.php");

$plot_colors = array('orange', 'blue', 'yellow', 'green', 'red', 'cyan');


$session = Session::getSession();
$Lang = $session->getLanguage();

$Project = GFUrl::getUrl()->getProject();
$project_id = $Project->getProjectId();

$con = Propel::getConnection(TrackerPeer::DATABASE_NAME);
$stmt = $con->createStatement();

$subprojects=((isset($_REQUEST['subprojects'])) ? Validator::sanitizeInt($_REQUEST['subprojects']) : 0 );
// Get all valid Project Ids
$all_ids = array();
if ($subprojects == 1) {
	$all_ids = $Project->getAllSubprojectIds();
	$all_ids[] = $project_id;
} else {
	$all_ids[] = $project_id;
}

if (!isset($_GET['tracker_id']) || $_GET['tracker_id'] == 0) {
	$sqlaux = " t.project_id IN (".implode(',',$all_ids).") ";
} else {
	$sqlaux = " t.tracker_id = '".$_GET['tracker_id']."'";
}

$sqlmine = '';
$sqlmytable = '';
$user_id=((isset($_REQUEST['user_id'])) ? Validator::sanitizeInt($_REQUEST['user_id']) : 0 );
$priority=((isset($_REQUEST['priority'])) ? Validator::sanitizeInt($_REQUEST['priority']) : 0 );
$status = ((isset($_REQUEST['status'])) ? Validator::sanitizeInt($_REQUEST['status']) : 0 );

if ($user_id) {
	$sqlmine = "AND tia.tracker_item_id=ti.tracker_item_id
	AND (ti.submitted_by = $user_id
	OR tia.assignee=$user_id)";

	$sqlmytable = ', tracker_item_assignee tia ';
}

// If there's a valid project_id, we'll filter by project all the queries


if ($priority)	{
	$sqlmine .= " AND ti.priority = $priority";	
}

if ($status > 0)	{
	$sqlmine .= " AND ti.status_id = " . ($status - 1);	
}

//Build an array of the last 18 months in YYYY-MM format
$startdate=(time()-1.5*365*60*60*24);
$addmonth=intval(date('m',$startdate));
$startyear=intval(date('Y',$startdate));
$addyear=0;

$users = array();


$c = new Criteria();
$c->add(AuditTrailPeer::SECTION, TrackerItem::SECTION);
$c->addOr(AuditTrailPeer::FIELD_NAME, 'assignee');
$c->addOr(AuditTrailPeer::FIELD_NAME, 'status');

//$c->add(AuditTrailPeer::CHANGE_DATE, $startdate, Criteria::GREATER_EQUAL);
$c->addJoin(AuditTrailPeer::REF_ID, TrackerItemPeer::TRACKER_ITEM_ID);

if($priority) {
	$c->add(TrackerItemPeer::PRIORITY, $priority);
}
if($status>0) {
	$c->add(TrackerItemPeer::STATUS_ID, $status -1 );
}

$c->addJoin(TrackerItemPeer::TRACKER_ID, TrackerPeer::TRACKER_ID);
foreach ($all_ids as $prjid) {
	$c->addAnd(TrackerPeer::PROJECT_ID, $prjid);
}

$c->addAscendingOrderByColumn(AuditTrailPeer::CHANGE_DATE);

$trails = AuditTrailPeer::doSelect($c);

// Process the trails and accumulate the results per user.
foreach ($trails as $trail) {
	// Process assignee changes
	$user = $trail->getNewValue();

	if ($trail->getFieldName() == 'assignee') {
		if(!isset($users[$user])) {
			$users[$user] = array('items'=>array());
		}

		$date = $trail->getChangeDate('Y-m');
		if(!isset($users[$user][$date])) {
			$users[$user][$date] = 0;
		}

		if (strcasecmp($user,'None') != 0 and strcasecmp($user,'Removed') != 0) {
			// User was assigned an item
			$users[$user][$date]++;
			if(!isset($users[$user]['items'])) {
				$users[$user]['items'] = array();
			}
			$users[$user]['items'][$trail->getRefId()]=1;
		}
		else {
			if(!isset($users[$trail->getOldValue()])) {
				$users[$trail->getOldValue()] = array();
			}
			if(!isset($users[$trail->getOldValue()][$date])) {
				$users[$trail->getOldValue()][$date] = 0;
			}
			$users[$trail->getOldValue()][$date]--;
			if(!isset($users[$user]['items'])) {
				$users[$user]['items'] = array();
			}
			if(isset($users[$user]['items'][$trail->getRefId()])) {
				unset ($users[$user]['items'][$trail->getRefId()]);
			}
		}
	} else if ($trail->getFieldName() == 'status') {
		$date = $trail->getChangeDate('Y-m');
		// Substract from the count if an item has been closed
		if($trail->getNewValue() == 'Closed') {
			foreach($users as $user => $data) {
				if(isset($users[$user]['items'][$trail->getRefId()])) {
					if(!isset($users[$user][$date])) {
						$users[$user][$date] = 0;
					}
					$users[$user][$date]--;
				}
			}
		}
		else if($trail->getNewValue() == 'Open') {
		// Item has been opened, maybe again after being close, increase the count.
			foreach($users as $user => $data) {
				if(isset($users[$user]['items'][$trail->getRefId()])) {
					if(!isset($users[$user][$date])) {
						$users[$user][$date] = 0;
					}
					$users[$user][$date]++;
				}
			}
		}
	}
}
unset ($users['Removed']);
unset ($users['Nobody User']);

// Integrate each month with the previous months

foreach ($users as $user => $data) {
	$last_date = '';
	unset($data['items']);
	foreach ($data as $date => $count) {

		if ($last_date != '') {
			$users[$user][$date] += $users[$user][$last_date];
		}
		$last_date = $date;
	}
}

foreach($users as $user => $data)
{
	$addmonth=intval(date('m',$startdate));
	$startyear=intval(date('Y',$startdate));
	$addyear=0;	
	
	$old_data = $data;
	$users[$user] = array();

	for ($i=0;$i<19;$i++) {
		$month = "".($startyear+$addyear).'-'.(sprintf("%02u",$addmonth))."";

		if(isset($old_data[$month])) {
			$users[$user][$month] = $old_data[$month];
		}
		else {
			$users[$user][$month] = 0;
		}
		
		$addmonth++;
		if ($addmonth > 12) {
			$addmonth=1;
			$addyear++;
		}
	}
	unset($old_data);

	if(!isset($months)) {
		$months = array_keys($users[$user]);
	}
}
$font = $Lang->getFontProperties(FF_FONT1, FS_NORMAL, 12);


//Build an array of the last 18 months in YYYY-MM format
if(!(isset($months) && count($months)>0)) {
	$notEnoughDataString = $Lang->getText('ProjectReporting.NotEnoughDataToPlot');
	$months=array();
	$startdate=(time()-1.5*365*60*60*24);
	$addmonth=intval(date('m',$startdate));
	$startyear=intval(date('Y',$startdate));
	$addyear=0;
	$months["".($startyear+$addyear).'-'.($addmonth).""]=0;
	for ($i=0; $i<18; $i++) {
		$addmonth++;
		if ($addmonth > 12) {
			$addmonth=1;
			$addyear++;
		}
		$months[]="".($startyear+$addyear).'-'.($addmonth)."";
		$users[$notEnoughDataString][$months[$i]]=0;
	}
	
	$plot_colors = array('white');
}

if(isset($months) && count($months)>0) {

	// Create the graph. These two calls are always required
	$graph = new Graph(600, 250);
	$graph->title->Set($Lang->getText('TrackerReporting.ItemsByAssignee'));
	$graph->title->SetFont($font[0], $font[1], $font[2]);
	$graph->SetScale('textlin');
	$graph->setBox(false);
	$graph->setFrame(false);
	$graph->img->SetMargin(40,200,30,50);
	$graph->yaxis->scale->SetGrace(0);
	$graph->yaxis->SetLabelMargin(1);
	$graph->yaxis->SetTitle($Lang->getText('TrackerReporting.Items'), 'low');
	$graph->yaxis->title->SetFont(FF_FONT1,FS_BOLD);
	$graph->yaxis->SetTitleMargin(30);
	$graph->xaxis->SetTickLabels($months);
	$graph->xaxis->SetLabelAngle(90);

	//print_r($users);

	$color_counter = 0;

	foreach ($users as $user => $data) {
		if (strlen($user) > 22) {
			$user_1 = substr($user, 0, 21);
			$user_2 = substr($user, 21, strlen($user));
			$user = $user_1."\n".$user_2;
		}
		
		$l = new LinePlot(array_values($data));

		$l->value->Show();
		$l->value->SetFormat('%d');
		$l->SetLegend($user);
		$l->SetWeight(3);
		$l->SetColor($plot_colors[$color_counter]);
		$graph->Add($l);

		$color_counter++;
		if($color_counter>=count($plot_colors)) {
			$color_counter=0;
		}
	}
}
else {
	// Create a not enough data information available graph

	$graph = new CanvasGraph(600, 250);

	$t1 = new Text($Lang->getText('ProjectReporting.NotEnoughDataToPlot') . "\n" . $Lang->getText('TrackerReporting.ItemsByAssignee'));
	$t1->SetPos(0.5,0.5);
	$t1->SetAlign('center', 'center', 'center');
	$t1->SetFont($font[0], $font[1], $font[2]);
	$t1->SetBox("white","black",true);
	$t1->SetColor("black");
	$graph->AddText($t1);
}

$graph->Stroke();
?>
 