<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id$
 * @author Ruben Gutierrez	ruben@gforgegroup.com
 * @date 2006-03-16
 *
 * SOAP plugin for GForge5.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
 
require_once('plugins/xmlcompatibility/lib/soap/artifact/ArtifactLogTypes.php');
require_once('plugins/tracker/lib/TrackerItemPeer.php');
require_once('lib/core/UserPeer.php');


$server->register(
	'artifactGetChangeLog',
	array(
		'session_ser'=>'xsd:string',
		'group_id'=>'xsd:int',
		'group_artifact_id'=>'xsd:int',
		'artifact_id' => 'xsd:int'
	),
	array('artifactGetChangeLogResponse'=>'tns:ArrayOfArtifactChangeLog'),
	$uri,
	$uri.'#artifactGetChangeLog','rpc','encoded'
);

function artifactGetChangeLog($session_ser, $group_id, $group_artifact_id, $artifact_id) {
	try {
		$session = continueSession($session_ser);
	
		$TrackerItem = TrackerItemPeer::retrieveByPk($artifact_id);
		if (!$TrackerItem || !is_object($TrackerItem)) {
			return new soap_fault('', 'artifactGetChangeLog', 'Could Not Get Change Log: No Such Artifact',
								'Could Not Get Change Log: No Such Artifact');
		}
		$Tracker=$TrackerItem->getTracker();
		//validate that this is a legitimate trackeritem for this tracker and user can add items
		if ($Tracker->getProject()->getProjectId() != $group_id || $Tracker->getTrackerId() != $group_artifact_id) {
			return new soap_fault('', 'artifactGetChangeLog', 'Could Not Get Change Log: Permission Denied',
								'Could Not Get Change Log: Permission Denied');
		}
	
		$c = new Criteria();
		$c->add(AuditTrailPeer::SECTION, TrackerItem::SECTION);
		$c->add(AuditTrailPeer::REF_ID, $TrackerItem->getTrackerItemId());
		$AuditTrails = AuditTrailPeer::doSelect($c);
		
		return auditTrailToSoap($AuditTrails, $Tracker);
	} catch (Exception $ex) {
		return new soap_fault ('', 'artifactGetChangeLog',
							'Could Not Get Change Log: ' . $ex->getMessage(),
							'Could Not Get Change Log: ' . $ex->getMessage());
	}
}

function auditTrailToSoap($AuditTrails, $Tracker) {
	$result = array();

	foreach ($AuditTrails as $item) {
		$field_name = $item->getFieldName();;
		$old_value = $item->getOldValue();
		$date = $item->getChangeDate(null);
		$user_name = $item->getUser()->getFullName();
		
		if ($field_name == 'status_id') {
			$old_value = $Tracker->getStatusName($old_value);
		} else if ($field_name == 'assigned_to') {
			$User = UserPeer::retrieveByUnixName($old_value);
			if ($User != null && is_object($User)) {
				$old_value =  $User->getFullName();
			}
		} else if ($field_name == 'close_date' || $field_name == 'open_date') {
			$old_value =  (is_numeric($old_value)) ? date('Y-m-d H:i:s', $old_value) : $old_value;
		}
		$result[] = array(
						"field_name"=> $field_name,
						"old_value"	=> $old_value,
						"date"		=> $date,
						"user_name"	=> $user_name
					);
	}
	
	return $result;
}

?>
