<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id$
 * @author Ruben Gutierrez	ruben@gforgegroup.com
 * @date 2006-03-16
 *
 * SOAP plugin for GForge5.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
 

require_once('plugins/tracker/lib/TrackerPeer.php');
require_once('plugins/tracker/lib/TrackerItemPeer.php');
require_once('lib/core/Monitor.php');

//
//	Artifact IsMonitorring
//
$server->register(
	'artifactIsMonitoring',
	array('session_ser'=>'xsd:string','group_id'=>'xsd:int','group_artifact_id'=>'xsd:int','artifact_id'=>'xsd:int'),
	array('artifactIsMonitoringResponse'=>'xsd:boolean'),
	$uri,$uri.'#artifactIsMonitoringResponse','rpc','encoded'
);

//
//	Artifact SetMonitor
//
$server->register(
	'artifactSetMonitor',
	array('session_ser'=>'xsd:string','group_id'=>'xsd:int','group_artifact_id'=>'xsd:int','artifact_id'=>'xsd:int'),
	array('artifactSetMonitorResponse'=>'xsd:boolean'),
	$uri,$uri.'#artifactSetMonitorResponse','rpc','encoded'
);

function artifactIsMonitoring($session_ser, $group_id, $group_artifact_id, $artifact_id) {
	try {
		$session = continueSession($session_ser);
	
		// Retrieve the Tracker Items
		$c = new Criteria();
		$c->add(TrackerItemPeer::TRACKER_ITEM_ID, $artifact_id);
		$c->add(TrackerItemPeer::TRACKER_ID, $group_artifact_id);
		// Do we really need this?
		$c->addJoin(TrackerPeer::PROJECT_ID, ProjectPeer::PROJECT_ID);
		$c->addJoin(TrackerPeer::TRACKER_ID, TrackerItemPeer::TRACKER_ID);
		$c->add(ProjectPeer::PROJECT_ID, $group_id);
		//
		$TrackerItem = TrackerItemPeer::doSelectOne($c);
		if (!$TrackerItem || !is_object($TrackerItem)) {
			return new soap_fault('', 'artifactSetMonitor', 'No Such Artifact',
								'No Such Artifact');
		}
	
		$user_id = $session->getUser()->getID();
		$c = new Criteria();
		$c->add(MonitorPeer::SECTION, TrackerItem::SECTION);
		$c->add(MonitorPeer::REF_ID, $artifact_id);
		$c->add(MonitorPeer::USER_ID, $user_id);
		$mon = MonitorPeer::doSelectOne($c);
	
		if (!is_null($mon)) {// new monitor
			return true;
		} else {
			return false;
		}
	} catch (Exception $ex) {
		return new soap_fault('', 'artifactIsMonitoring',
							'Could Not Get Artifact Is Monitoring: '.$ex->getMessage(),
							'Could Not Get Artifact Is Monitoring: '.$ex->getMessage());
	}
}

function artifactSetMonitor($session_ser, $group_id, $group_artifact_id, $artifact_id) {
	try {
		$session = continueSession($session_ser);
	
		// Retrieve the Tracker Items
		$c = new Criteria();
		$c->add(TrackerItemPeer::TRACKER_ITEM_ID, $artifact_id);
		$c->add(TrackerItemPeer::TRACKER_ID, $group_artifact_id);
		// Do we really need this?
		$c->addJoin(TrackerPeer::PROJECT_ID, ProjectPeer::PROJECT_ID);
		$c->addJoin(TrackerPeer::TRACKER_ID, TrackerItemPeer::TRACKER_ID);
		$c->add(ProjectPeer::PROJECT_ID, $group_id);
		//
		$TrackerItem = TrackerItemPeer::doSelectOne($c);
		if (!$TrackerItem || !is_object($TrackerItem)) {
			return new soap_fault('', 'artifactSetMonitor', 'No Such Artifact',
								'No Such Artifact');
		}
	
		$user_id = $session->getUser()->getID();
		$c = new Criteria();
		$c->add(MonitorPeer::SECTION, TrackerItem::SECTION);
		$c->add(MonitorPeer::REF_ID, $artifact_id);
		$c->add(MonitorPeer::USER_ID, $user_id);
		$mon = MonitorPeer::doSelectOne($c);
	
		if (is_null($mon)) {// new monitor
			$mon = new Monitor();
			$mon->setSection(TrackerItem::SECTION);
			$mon->setRefId($artifact_id);
			$mon->setUserId($user_id);
			try {
				$mon->save();
			} catch (InvalidDataException $ex) {
				return new soap_fault('', 'artifactSetMonitor',
								'Could Not Set Monitor: '.$ex->getMessage(),
								'Could Not Set Monitor: '.$ex->getMessage());
			}
	
		} else {
			$mon->delete();
		}
		return true;
	} catch (Exception $ex) {
		return new soap_fault('', 'artifactSetMonitor',
							'Could Not Set Monitor: '.$ex->getMessage(),
							'Could Not Set Monitor: '.$ex->getMessage());
	}
}
?>
