<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

// This code generates a SOAP API for handling FrsRelease

require_once('plugins/frs/lib/FrsRelease.php');
require_once('plugins/frs/lib/FrsReleasePeer.php');
require_once('Validator.php');

/* ******************************************************************** */
/*                                 DATA TYPES                           */
/* ******************************************************************** */
$server->wsdl->addComplexType(
	'FrsRelease',
	'complexType',
	'struct',
	'sequence',
	'',
	array(
		'frs_release_id' => array('name'=>'frs_release_id', 'type' => 'xsd:int'),
		'frs_package_id' => array('name'=>'frs_package_id', 'type' => 'xsd:int'),
		'release_name' => array('name'=>'release_name', 'type' => 'xsd:string'),
		'release_notes' => array('name'=>'release_notes', 'type' => 'xsd:string'),
		'changes' => array('name'=>'changes', 'type' => 'xsd:string'),
		'status_id' => array('name'=>'status_id', 'type' => 'xsd:int'),
		'preformatted' => array('name'=>'preformatted', 'type' => 'xsd:boolean'),
		'release_date' => array('name'=>'release_date', 'type' => 'xsd:string'),
		'released_by' => array('name'=>'released_by', 'type' => 'xsd:int'),
		'is_released' => array('name'=>'is_released', 'type' => 'xsd:boolean'),
	)
);


$server->wsdl->addComplexType(
	'ArrayOfFrsRelease',
	'complexType',
	'array',
	'',
	'SOAP-ENC:Array',
	array(),
	array(array('ref' => 'SOAP-ENC:arrayType', 'wsdl:arrayType' => 'tns:FrsRelease[]')),
	'tns:FrsRelease'
);

/* ******************************************************************** */
/*                                 METHODS                              */
/* ******************************************************************** */

$server->register(
	'getFrsReleases',
	array(
		'sessionHash'	=> 'xsd:string',
		'frs_package_id' => 'xsd:int',
		'released_by' => 'xsd:int'
	), 
	array('getFrsReleases' => 'tns:ArrayOfFrsRelease'),
	$uri, 
	$uri.'#getFrsReleases',
	'rpc',
	'encoded'
);


/**
 * SOAP method to retrieve a list of FrsRelease
 * @param	$frs_package_id	Filter by frs_package_id
 */
function getFrsReleases($sessionHash, $frs_package_id, $released_by) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($sessionHash, $ip);

	$c = new Criteria();
	$c->add(FrsReleasePeer::FRS_PACKAGE_ID, $frs_package_id);

	if ($released_by >= 0) {
		$c->add(FrsReleasePeer::RELEASED_BY, $released_by);
	}
	
	$objs = FrsReleasePeer::doSelect($c);
	$res = array();
	foreach ($objs as $obj) {
		if ($obj->userCanRead()) {
			$res[] = FrsReleaseToSOAP($obj);
		}
	}
	return $res;
}

$server->register(
	'getFrsReleaseArray',
	array(
		'sessionHash'	=> 'xsd:string',
		'frs_release_id' => 'tns:ArrayOfInt',
	), 
	array('getFrsRelease' => 'tns:ArrayOfFrsRelease'),
	$uri, 
	$uri.'#getFrsRelease',
	'rpc',
	'encoded'
);

/**
 * SOAP method to retrieve an array of FrsRelease
 * @param	$frs_release_id	Retrieve the FrsRelease array identified by the array of frs_release_id
 */
function getFrsReleaseArray($sessionHash, $frs_release_idArr) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($sessionHash, $ip);

	$ret = array();

	$crit=new Criteria();
        $crit->add(FrsReleasePeer::FRS_RELEASE_ID,$frs_release_idArr,Criteria::IN);
	$objs = FrsReleasePeer::doSelect($crit);
	foreach ($objs as $obj) {
		if ($obj->userCanRead()) {
			$ret[] = FrsReleaseToSOAP($obj);
		}
	}
	return $ret;
}

$server->register(
	'getFrsRelease',
	array(
		'sessionHash'	=> 'xsd:string',
		'frs_release_id' => 'xsd:int',
	), 
	array('getFrsRelease' => 'tns:FrsRelease'),
	$uri, 
	$uri.'#getFrsRelease',
	'rpc',
	'encoded'
);

/**
 * SOAP method to retrieve a single FrsRelease
 * @param	$frs_release_id	Retrieve the FrsRelease identified by this frs_release_id
 */
function getFrsRelease($sessionHash, $frs_release_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($sessionHash, $ip);

	$obj = FrsReleasePeer::retrieveByPK($frs_release_id);
	if (is_null($obj)) {
		return null;
	}
	if (!$obj->userCanRead()) {
		return new soap_fault('', 'getFrsRelease','Permission Denied - Cannot View This Record','Permission Denied - Cannot View This Record');
	}
	return FrsReleaseToSOAP($obj);
}


$server->register(
	'deleteFrsRelease',
	array(
		'sessionHash'	=> 'xsd:string',
		'frs_release_id' => 'xsd:int',
	), 
	array('deleteFrsRelease' => 'xsd:int'),
	$uri, 
	$uri.'#deleteFrsRelease',
	'rpc',
	'encoded'
);

/**
 * SOAP method to delete a single FrsRelease
 * @param	$frs_release_id	Delete the FrsRelease identified by this frs_release_id
 */
function deleteFrsRelease($sessionHash, $frs_release_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($sessionHash, $ip);

	$obj = FrsReleasePeer::retrieveByPK($frs_release_id);
	if (is_null($obj)) {
		return new soap_fault('', 'deleteFrsRelease','Permission Denied - Cannot Delete This Record','Permission Denied - Cannot View This Record');
	}
	try {
		$obj->delete();
	} catch (Exception $e) {
		return new soap_fault('', 'deleteFrsRelease','Permission Denied - Cannot Delete This Record',$e->getMessage());
	}
	return 1;
}


$server->register(
	'updateFrsRelease',
	array(
		'sessionHash'	=> 'xsd:string',
		'frs_release_id' => 'xsd:int',
		'release_name' => 'xsd:string',
		'release_notes' => 'xsd:string',
		'changes' => 'xsd:string',
		'status_id' => 'xsd:int',
		'preformatted' => 'xsd:boolean',
		'release_date' => 'xsd:string',
		'is_released' => 'xsd:boolean',
	), 
	array('updateFrsRelease' => 'xsd:int'),
	$uri, 
	$uri.'#updateFrsRelease',
	'rpc',
	'encoded'
);

/**
 * SOAP method to update FrsRelease
 * @param	$release_name
 * @param	$release_notes
 * @param	$changes
 * @param	$status_id
 * @param	$preformatted
 * @param	$release_date
 */
function updateFrsRelease($sessionHash, $frs_release_id, $release_name, $release_notes, $changes, $status_id, $preformatted, $release_date, $is_released=true) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($sessionHash, $ip);

	$obj = FrsReleasePeer::retrieveByPK($frs_release_id);
	if (is_null($obj)) {
		return new soap_fault('', 'updateFrsRelease','Couldn\'t update FrsRelease: invalid frs_release_id '.$frs_release_id);
	}
	
	if (!is_null($release_name)) {
		$obj->setReleaseName( Validator::sanitizeString($release_name) );
	}
	if (!is_null($release_notes)) {
		$obj->setReleaseNotes( Validator::sanitizeString($release_notes) );
	}
	if (!is_null($changes)) {
		$obj->setChanges( Validator::sanitizeString($changes) );
	}
	if (!is_null($status_id)) {
		$obj->setStatusId( Validator::sanitizeInt($status_id) );
	}
	if (!is_null($preformatted)) {
		$obj->setPreformatted( Validator::sanitizeBool($preformatted) );
	}
	if (!is_null($release_date)) {
		$obj->setReleaseDate( Validator::sanitizeDate($release_date) );
	}
	if (!is_null($is_released)) {
		$obj->setIsReleased( Validator::sanitizeBool($preformatted) );
	}
	
	try {
		$obj->save();
	} catch (Exception $e) {
		return new soap_fault('', 'updateFrsRelease','Permission Denied - Cannot update This Record',$e->getMessage());
	}
	return 1;
}



$server->register(
	'addFrsRelease',
	array(
		'sessionHash'	=> 'xsd:string',
		'frs_package_id' => 'xsd:int',
		'release_name' => 'xsd:string',
		'release_notes' => 'xsd:string',
		'changes' => 'xsd:string',
		'status_id' => 'xsd:int',
		'preformatted' => 'xsd:boolean',
		'release_date' => 'xsd:string',
		'is_released' => 'xsd:boolean',
	), 
	array('addFrsRelease' => 'xsd:int'),
	$uri, 
	$uri.'#getFrsReleases',
	'rpc',
	'encoded'
);


/**
 * SOAP method to add a FrsRelease
 * @param	$frs_package_id
 * @param	$release_name
 * @param	$release_notes
 * @param	$changes
 * @param	$status_id
 * @param	$preformatted
 * @param	$release_date
 */
function addFrsRelease($sessionHash, $frs_package_id, $release_name, $release_notes, $changes, $status_id, $preformatted, $release_date, $is_released=true) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($sessionHash, $ip);

	$frsPackage = FrsPackagePeer::retrieveByPK($frs_package_id);
	if (is_null($frsPackage)){
		return new soap_fault('', 'addFrsRelease','Permission Denied - Cannot add This Record', 'Permission Denied - Cannot add This Record');
	}

	$project = $frsPackage->getProject();
	if (is_null($project) || !$project->getPermission()->check(FrsPlugin::SECTION,0,FrsPlugin::WRITE)) {
		return new soap_fault('', 'addFrsRelease','Permission Denied - Cannot add This Record', 'Permission Denied - Cannot add This Record');
	}

	$obj = new FrsRelease();
	
	$obj->setFrsPackageId( Validator::sanitizeInt($frs_package_id) );
	$obj->setReleaseName( Validator::sanitizeString($release_name) );
	$obj->setReleaseNotes( Validator::sanitizeString($release_notes) );
	$obj->setChanges( Validator::sanitizeString($changes) );
	$obj->setStatusId( Validator::sanitizeInt($status_id) );
	$obj->setPreformatted( Validator::sanitizeBool($preformatted) );
	$obj->setReleaseDate( Validator::sanitizeDate($release_date) );
	$obj->setIsReleased( Validator::sanitizeBool($is_released) );
	$obj->setReleasedBy( Session::getSession()->getUser()->getUserId() );
	try {
		$obj->save();
		return $obj->getFrsReleaseId();
	} catch (Exception $e) {
		return new soap_fault('', 'addFrsRelease','Permission Denied - Cannot add This Record',$e->getMessage());
	}
}

function FrsReleaseToSOAP(&$obj) {
	$ret = array();
	$ret['frs_release_id'] = $obj->getFrsReleaseId();
	$ret['frs_package_id'] = $obj->getFrsPackageId();
	$ret['release_name'] = $obj->getReleaseName();
	$ret['release_notes'] = $obj->getReleaseNotes();
	$ret['changes'] = $obj->getChanges();
	$ret['status_id'] = $obj->getStatusId();
	$ret['preformatted'] = $obj->getPreformatted();
	$ret['release_date'] = $obj->getReleaseDate();
	$ret['released_by'] = $obj->getReleasedBy();
	$ret['is_released'] = $obj->getIsReleased();
	return $ret;
}
?>
