<?php

/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id$
 * @author Julian Yerfino jyerfino@gforgegroup.com
 * @date
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('html/MyTemplate.php');
require_once('lib/core/FilesystemPeer.php');
require_once('lib/core/MyReport.php');
require_once('lib/core/MyReportPeer.php');
require_once('lib/core/MyReportFieldFilter.php');
require_once('lib/core/MyReportFieldDisplay.php');
require_once('lib/core/MyReportFieldFilterPeer.php');
require_once('lib/core/MyReportFieldDisplayPeer.php');
require_once('lib/core/MyReportTracker.php');
require_once('lib/core/MyReportTrackerPeer.php');
require_once('lib/core/MyReportProject.php');
require_once('lib/core/MyReportProjectPeer.php');
require_once('lib/core/MyReportFieldValue.php');
require_once('lib/core/MyReportFieldValuePeer.php');
require_once 'lib/external/html2pdf/HTML_ToPDF.php';

ob_start();

$session=Session::getSession();
if (!$session->isLoggedIn()) {
	throw new NotLoggedInException();
}
$template = new MyTemplate();
$user = Session::getSession()->getUser();

if(isset($_REQUEST['feedback'])) $feedback = $_REQUEST['feedback'];
if(isset($_REQUEST['report_id'])) {
	$report_id = Validator::sanitizeInt($_REQUEST['report_id']);
	// Fetch report data
	if($report_id != 0) {
		$report = MyReportPeer::retrieveByPK($report_id);
		if(!is_object($report)) {
			throw new InvalidDataException();
		}
		// Check if the user can view the report
		if (!$report->userCanRead()) {
			throw new PermissionDeniedException();
		}
	}
	else {
		throw new InvalidDataException();
	}
}
else {
	throw new InvalidDataException();
}
$Lang = $session->getLanguage();

if(isset($_REQUEST['feedback'])) $feedback = $_REQUEST['feedback'];
$params = array(
	'title' => Language::getSessText('Report.ViewReport'),
	'feedback' => isset($_REQUEST['feedback']) ? $_REQUEST['feedback'] : (isset($feedback) ? $feedback : ''),
	'action' => isset($action) ? $action : '',
	'selectedbottomtab' => 'myreports',
	'user' => $user
	);
$params['theme'] = $template;
	
//$template->header($params);

$tracker_item_ids = $report->getTrackerItemIds();
$theader = array();
$theader[] = Language::getSessText('Report.TrackerItemId');

if(!is_array($tracker_item_ids) || count($tracker_item_ids)==0) {
	echo $theme->tableTop($theader);
	echo $theme->tableFullRow(Language::getSessText('Report.NoMatchesFound'));
}
else {
	//$tracker_items = TrackerItemPeer::retrieveByPKs($tracker_item_ids);
	$display_data = $report->getDisplayData($tracker_item_ids);
	
	$crit = new Criteria();
	$crit->add(MyReportFieldDisplayPeer::MY_REPORT_ID, $report_id);
	//$crit->addAscendingOrderByColumn(MyReportFieldDisplayPeer::FIELD_NAME);
	$display_field_names = MyReportFieldDisplayPeer::doSelect($crit);
	$headers = array();
	foreach($display_field_names as $display_field_name) {
		switch($display_field_name->getType()) {
			case MyReport::FIELDTYPE_ASSIGNEES:
				$theader[] = Language::getSessText('TrackerItem.AssignedTo');
				break;
			case MyReport::FIELDTYPE_SUMMARY:
				$theader[] = Language::getSessText('TrackerItem.SummaryName');
				break;
			case MyReport::FIELDTYPE_DETAILS:
				$theader[] = Language::getSessText('TrackerItem.DetailsName');
				break;
			case MyReport::FIELDTYPE_PRIORITY:
				$theader[] = Language::getSessText('TrackerItem.PriorityName');
				break;	
			case MyReport::FIELDTYPE_OPENDATE:
				$theader[] = Language::getSessText('TrackerItem.OpenDateName');
				break;
			case MyReport::FIELDTYPE_CLOSEDATE:
				$theader[] = Language::getSessText('TrackerItem.CloseDateName');
				break;
			case MyReport::FIELDTYPE_LASTMODIFIEDDATE:
				$theader[] = Language::getSessText('TrackerItem.LastModifiedDateName');
				break;
			case MyReport::FIELDTYPE_TRACKERNAME:
				$theader[] = Language::getSessText('Tracker.TabString');
				break;
			case MyReport::FIELDTYPE_PROJECTNAME:
				$theader[] = Language::getSessText('Common.Project');
				break;		
			default:
				$theader[] = $display_field_name->getFieldName();
				break;
		}
		$headers[] = $display_field_name->getFieldName();
	}

	echo $theme->tableTop($theader,false, 'bgcolor="#4D4D74"');
	
	
	
	$tmod = array();

	foreach($display_data as $tracker_item_id => $display_fields) {
		$trow = array();
		
		$tracker_item = TrackerItemPeer::retrieveByPk($tracker_item_id);
		$trow[] = '<a href="'. GFUrl::getUrl()->projectURL($tracker_item->getTracker()->getProject()->getUnixName(), 'TrackerItemEdit', 'tracker', '', array('tracker_item_id' => $tracker_item_id)) .'" title="'. $tracker_item->getSummary() .'">'. $tracker_item_id .'</a>';
		
		foreach($headers as $header) {
			if(isset($display_fields[$header]) && count($display_fields[$header])>0) {
				$trow_str = '';
				foreach($display_fields[$header] as $row_str ) {
					$trow_str .= "$row_str<br>";
				}
				$trow[] = $trow_str;
			}
			else {
				$trow[] = '&nbsp;';
			}
		}
		echo $theme->tableRow($trow, $tmod, true);
	}
}

echo $theme->tableBottom();

$template->footer();


// Write output to a temp file and convert it to PDF
$html = ob_get_clean();
$htmlPath = tempnam('/tmp', 'gforge-pdf');
$htmlFp = fopen($htmlPath, 'w');
fputs($htmlFp, $html);
fclose($htmlFp);
$pdfPath = tempnam('/tmp', 'gforge-pdf');
#$pdfPath = SystemUtils::buildFilePath('/tmp',str_replace(' ', '_', $report->getReportName()) . ".pdf");
$pdf =& new HTML_ToPDF($htmlPath, GFConf::get('system.hostName'), $pdfPath);
//$pdf->setDebug(true);

$pdf->setHeader('left', $report->getReportName());
$pdf->setFooter('left', Language::getSessText('Report.MadeWithGForge'));
$pdf->setFooter('right', '$D');

// Set that we do not want to use the page's css
$pdf->setUseCSS(false);
// Give it our own css, in this case it will make it so
// the lines are double spaced
$pdf->setAdditionalCSS("
.tabular { /*float: left;*/ font-size: 11px; color: #5a5a89; background: #dddddd; }
.tabular td { padding: 3px; }
.tabular th a { color: #ffffff; font-size: x-small; }
tr.th th { text-align: left; padding: 0 9px; line-height:22px; color: #fff; background: #4D4D74 url('images/purple_th.gif') repeat-x; }
.noLeft { padding-left: 0px!important; }
.noRight { padding-right: 0px!important; }
.th_l { padding-left: 9px; background: url('images/purple_th_l.gif') top left no-repeat;  }
.th_r { padding-right: 9px; background: url('images/purple_th_r.gif') top right no-repeat;  }
tr.l { background: #fff; }
tr.d { background: #ededed; } /* was #F5F5F5 */
td.tFooter { padding: 0px; background: #e0e0e0; line-height:6px; }
");
// We want to underline links
$pdf->setUnderlineLinks(true);
// Scale the page down slightly
$pdf->setScaleFactor('1.2');
// Make the page black and light
$pdf->setUseColor(true);
// Convert the file
$result = $pdf->convert();

// Check if the result was an error
if (is_a($result, 'HTML_ToPDFException')) {
    die($result->getMessage());
}

// Fetch the file and send it to the user

header('Content-Length: ' . filesize($pdfPath));
header('Content-type: application/pdf');
#header('Content-Disposition: attachment; filename="' . basename($pdfPath) . '"');
header('Content-Disposition: attachment; filename="' . str_replace(' ', '_', $report->getReportName()) . ".pdf" . '"');

header('Content-Transfer-Encoding: binary');
header('Cache-Control: must-revalidate, post-check=0, pre-check=0');
header('Pragma: public');
header('Expires: 0');
header('Content-Description: ' . $report->getReportName());

flush();

# begin download
try {
	readfile($pdfPath);
}
catch (Exception $e){
	
}

@unlink($htmlPath);
@unlink($pdfPath);

exit();
?>
